@@ -1,5 +1,8 @@
 # mypy: disallow-untyped-defs
 """Invoke development tasks."""
+
+from __future__ import annotations
+
 import argparse
 import os
 from pathlib import Path
@@ -107,7 +110,7 @@ def pre_release(
 
 def changelog(version: str, write_out: bool = False) -> None:
     addopts = [] if write_out else ["--draft"]
-    check_call(["towncrier", "--yes", "--version", version] + addopts)
+    check_call(["towncrier", "--yes", "--version", version, *addopts])
 
 
 def main() -> None:
