@@ -1,3 +1,6 @@
+# mypy: allow-untyped-defs
+from __future__ import annotations
+
 import io
 import os
 from pathlib import Path
@@ -5,7 +8,6 @@ import re
 import shutil
 import sys
 from typing import Generator
-from typing import Optional
 from unittest import mock
 
 from _pytest._io import terminalwriter
@@ -165,7 +167,7 @@ def test_attr_hasmarkup() -> None:
     assert "\x1b[0m" in s
 
 
-def assert_color(expected: bool, default: Optional[bool] = None) -> None:
+def assert_color(expected: bool, default: bool | None = None) -> None:
     file = io.StringIO()
     if default is None:
         default = not expected
